import { Server } from "socket.io";
import {TSocket} from "../../utilities/Types";
import ZoneRepository from "../../repositories/ZoneRepository";
import ZoneManager from "../../ZoneManager";
import {Character, Zone} from "@prisma/client";

interface IPayload {
    zoneId: number;
}

/**
 * Handle character zone request event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
    socket.on('character:zone:request', async (data: IPayload) => {
        console.log(`---User ${socket.character?.id} has requested zone.`);

        if (!data.zoneId) {
            console.log(`---Zone id not provided.`);
            return;
        }

        const zone = await ZoneRepository.getById(data.zoneId);

        if (!zone) {
            console.log(`---Zone not found.`);
            return;
        }

        socket.join(zone.id.toString());

        // send over zone and characters to socket
        socket.emit('character:zone:load', {
            zone: zone,
            characters: ZoneManager.getCharactersInZone(zone.id),
        });

        // let other clients know of new character
        io.to(zone.id.toString()).emit('zone:character:join', socket.character);

        // add character to zone manager
        ZoneManager.addCharacterToZone(zone.id, socket.character as Character);
    });
}

/**
 * Resources:
 * https://stackoverflow.com/questions/6873607/socket-io-rooms-difference-between-broadcast-to-and-sockets-in
 */